/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "HTMLDocument.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomDocument.h"
#include "gdom/GdomHTMLDocument.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomDocument.h"
#include "gdom/PrivateGdomHTMLDocument.h"
#include "gdom/PrivateIntGdomHTMLDocument.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
} /* namespace WebCore */
extern "C" {

GDOMHTMLDocument::GDOMHTMLDocument(PassRefPtr<WebCore::HTMLDocument> impl)
    : GDOMDocument(impl)
{
}


/* XXX TODO: custom function gdom_html_document_open */

void
gdom_html_document_close (GdomHTMLDocument *thiz)
{
    g_return_if_fail (thiz);
    WebCore::HTMLDocument * item = (gdom_html_document_get_private(thiz));
    g_return_if_fail (item);
    item->close();

}


/* XXX TODO: custom function gdom_html_document_write */


/* XXX TODO: custom function gdom_html_document_writeln */

void
gdom_html_document_clear (GdomHTMLDocument *thiz)
{
    g_return_if_fail (thiz);
    WebCore::HTMLDocument * item = (gdom_html_document_get_private(thiz));
    g_return_if_fail (item);
    item->clear();

}

void
gdom_html_document_capture_events (GdomHTMLDocument *thiz)
{
    g_return_if_fail (thiz);
    WebCore::HTMLDocument * item = (gdom_html_document_get_private(thiz));
    g_return_if_fail (item);
    item->captureEvents();

}

void
gdom_html_document_release_events (GdomHTMLDocument *thiz)
{
    g_return_if_fail (thiz);
    WebCore::HTMLDocument * item = (gdom_html_document_get_private(thiz));
    g_return_if_fail (item);
    item->releaseEvents();

}

gboolean
gdom_html_document_has_focus (GdomHTMLDocument *thiz)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::HTMLDocument * item = (gdom_html_document_get_private(thiz));
    g_return_val_if_fail (item, 0);
    gboolean res = item->hasFocus();
    return res;

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "HTMLDocument.h"
#include <gdom/GdomHTMLDocument.h>
#include <gdom/PrivateGdomHTMLDocument.h>
#include <gdom/PrivateIntGdomHTMLDocument.h>
#include <gdom/GdomHTMLDocumentprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomHTMLDocument, gdom_html_document, GDOM_TYPE_DOCUMENT);

/*
struct _GdomHTMLDocumentPrivate {
    class WebCore::HTMLDocument *item;
};

#define GDOM_HTML_DOCUMENT_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_HTML_DOCUMENT, GdomHTMLDocumentPrivate))
*/
G_END_DECLS
WebCore::HTMLDocument *
gdom_html_document_get_private(GdomHTMLDocument *request)
{
    GDOMHTMLDocument *res;
    res = static_cast<GDOMHTMLDocument*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::HTMLDocument*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_EMBEDS,
    PROP_PLUGINS,
    PROP_SCRIPTS,
    PROP_WIDTH,
    PROP_HEIGHT,
    PROP_DIR,
    PROP_DESIGN_MODE,
    PROP_COMPAT_MODE,
    PROP_ACTIVE_ELEMENT,
    PROP_BG_COLOR,
    PROP_FG_COLOR,
    PROP_ALINK_COLOR,
    PROP_LINK_COLOR,
    PROP_VLINK_COLOR,
};


static void gdom_html_document_finalize(GObject* object)
{
    /*GdomHTMLDocument* request = GDOM_HTML_DOCUMENT(object);*/
    /*GdomHTMLDocumentPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_DOCUMENT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_html_document_parent_class)->finalize(object);
}

static void gdom_html_document__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomHTMLDocument *self = GDOM_HTML_DOCUMENT(object);
    WebCore::HTMLDocument * gself = (gdom_html_document_get_private(self));
    switch (prop_id) {
     case PROP_DIR:
    {
         gself->setDir(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_DESIGN_MODE:
    {
         gself->setDesignMode(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_BG_COLOR:
    {
         gself->setBgColor(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_FG_COLOR:
    {
         gself->setFgColor(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_ALINK_COLOR:
    {
         gself->setAlinkColor(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_LINK_COLOR:
    {
         gself->setLinkColor(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_VLINK_COLOR:
    {
         gself->setVlinkColor(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_html_document__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomHTMLDocument *self = GDOM_HTML_DOCUMENT(object);
    WebCore::HTMLDocument * gself = (gdom_html_document_get_private(self));
    switch (prop_id) {
    case PROP_EMBEDS:
    {
         PassRefPtr<WebCore::HTMLCollection> ptr = gself->embeds();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_PLUGINS:
    {
         PassRefPtr<WebCore::HTMLCollection> ptr = gself->plugins();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_SCRIPTS:
    {
         PassRefPtr<WebCore::HTMLCollection> ptr = gself->scripts();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_WIDTH:
    {
         g_value_set_long (value, gself->width());
         break;
    }
    case PROP_HEIGHT:
    {
         g_value_set_long (value, gself->height());
         break;
    }
    case PROP_DIR:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->dir()));
         break;
    }
    case PROP_DESIGN_MODE:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->designMode()));
         break;
    }
    case PROP_COMPAT_MODE:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->compatMode()));
         break;
    }
    case PROP_ACTIVE_ELEMENT:
    {
         PassRefPtr<WebCore::Element> ptr = gself->activeElement();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_BG_COLOR:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->bgColor()));
         break;
    }
    case PROP_FG_COLOR:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->fgColor()));
         break;
    }
    case PROP_ALINK_COLOR:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->alinkColor()));
         break;
    }
    case PROP_LINK_COLOR:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->linkColor()));
         break;
    }
    case PROP_VLINK_COLOR:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->vlinkColor()));
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_html_document_class_init(GdomHTMLDocumentClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_html_document_finalize;
    g_klass->set_property = gdom_html_document__set_property_fn;
    g_klass->get_property = gdom_html_document__get_property_fn;

    g_object_class_install_property (g_klass, PROP_EMBEDS,
            g_param_spec_object ("embeds", /* name */
                "html_document_embeds", /* short description */
                "read-only  GdomHTMLCollection * HTMLDocument.embeds", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_HTML_COLLECTION, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_PLUGINS,
            g_param_spec_object ("plugins", /* name */
                "html_document_plugins", /* short description */
                "read-only  GdomHTMLCollection * HTMLDocument.plugins", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_HTML_COLLECTION, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_SCRIPTS,
            g_param_spec_object ("scripts", /* name */
                "html_document_scripts", /* short description */
                "read-only  GdomHTMLCollection * HTMLDocument.scripts", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_HTML_COLLECTION, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_WIDTH,
            g_param_spec_long ("width", /* name */
                "html_document_width", /* short description */
                "read-only  glong HTMLDocument.width", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_HEIGHT,
            g_param_spec_long ("height", /* name */
                "html_document_height", /* short description */
                "read-only  glong HTMLDocument.height", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_DIR,
            g_param_spec_string ("dir", /* name */
                "html_document_dir", /* short description */
                "read-write  gchar * HTMLDocument.dir", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_DESIGN_MODE,
            g_param_spec_string ("design_mode", /* name */
                "html_document_design_mode", /* short description */
                "read-write  gchar * HTMLDocument.design_mode", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_COMPAT_MODE,
            g_param_spec_string ("compat_mode", /* name */
                "html_document_compat_mode", /* short description */
                "read-only  gchar * HTMLDocument.compat_mode", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_ACTIVE_ELEMENT,
            g_param_spec_object ("active_element", /* name */
                "html_document_active_element", /* short description */
                "read-only  GdomElement * HTMLDocument.active_element", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_ELEMENT, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_BG_COLOR,
            g_param_spec_string ("bg_color", /* name */
                "html_document_bg_color", /* short description */
                "read-write  gchar * HTMLDocument.bg_color", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_FG_COLOR,
            g_param_spec_string ("fg_color", /* name */
                "html_document_fg_color", /* short description */
                "read-write  gchar * HTMLDocument.fg_color", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_ALINK_COLOR,
            g_param_spec_string ("alink_color", /* name */
                "html_document_alink_color", /* short description */
                "read-write  gchar * HTMLDocument.alink_color", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_LINK_COLOR,
            g_param_spec_string ("link_color", /* name */
                "html_document_link_color", /* short description */
                "read-write  gchar * HTMLDocument.link_color", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_VLINK_COLOR,
            g_param_spec_string ("vlink_color", /* name */
                "html_document_vlink_color", /* short description */
                "read-write  gchar * HTMLDocument.vlink_color", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomHTMLDocumentPrivate));*/
}

static void gdom_html_document_init(GdomHTMLDocument* request)
{
    /*GdomHTMLDocumentPrivate* priv = GDOM_HTML_DOCUMENT_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

